'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const toSnakeCaseKeys = require('../_chunk/toSnakeCaseKeys-BNpS71.js');

function mergeWith(target, source, merge) {
    const sourceKeys = Object.keys(source);
    for (let i = 0; i < sourceKeys.length; i++) {
        const key = sourceKeys[i];
        const sourceValue = source[key];
        const targetValue = target[key];
        const merged = merge(targetValue, sourceValue, key, target, source);
        if (merged != null) {
            target[key] = merged;
        }
        else if (Array.isArray(sourceValue)) {
            target[key] = mergeWith(targetValue ?? [], sourceValue, merge);
        }
        else if (toSnakeCaseKeys.isObjectLike(targetValue) && toSnakeCaseKeys.isObjectLike(sourceValue)) {
            target[key] = mergeWith(targetValue ?? {}, sourceValue, merge);
        }
        else if (targetValue === undefined || sourceValue !== undefined) {
            target[key] = sourceValue;
        }
    }
    return target;
}

function omit(obj, keys) {
    const result = { ...obj };
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        delete result[key];
    }
    return result;
}

function pick(obj, keys) {
    const result = {};
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        if (Object.hasOwn(obj, key)) {
            result[key] = obj[key];
        }
    }
    return result;
}

function pickBy(obj, shouldPick) {
    const result = {};
    const keys = Object.keys(obj);
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        const value = obj[key];
        if (shouldPick(value, key)) {
            result[key] = value;
        }
    }
    return result;
}

exports.clone = toSnakeCaseKeys.clone;
exports.cloneDeep = toSnakeCaseKeys.cloneDeep;
exports.cloneDeepWith = toSnakeCaseKeys.cloneDeepWith;
exports.findKey = toSnakeCaseKeys.findKey;
exports.flattenObject = toSnakeCaseKeys.flattenObject;
exports.invert = toSnakeCaseKeys.invert;
exports.mapKeys = toSnakeCaseKeys.mapKeys;
exports.mapValues = toSnakeCaseKeys.mapValues;
exports.merge = toSnakeCaseKeys.merge;
exports.omitBy = toSnakeCaseKeys.omitBy;
exports.toCamelCaseKeys = toSnakeCaseKeys.toCamelCaseKeys;
exports.toMerged = toSnakeCaseKeys.toMerged;
exports.toSnakeCaseKeys = toSnakeCaseKeys.toSnakeCaseKeys;
exports.mergeWith = mergeWith;
exports.omit = omit;
exports.pick = pick;
exports.pickBy = pickBy;
