import os from 'os';
import constants from '../config/constants.js';
const normalizeOnlyFlagPairs = (flag1Key, flag2Key, opts, logger) => {
    const stripOnly = (key) => key.replace('Only', '');
    if (opts[flag1Key] && opts[flag2Key]) {
        logger.warn(`Hmm, you want to _only_ generate both ${stripOnly(flag1Key)} and ${stripOnly(flag2Key)} set. Ignoring --x-only settings as this is default behavior`);
        return {
            [flag1Key]: false,
            [flag2Key]: false,
        };
    }
    return {};
};
const normalizeOutput = (output) => {
    if (!output) {
        return '.';
    }
    return output;
};
const getDefaultOptions = () => {
    const flags = constants.FLAGS;
    // TODO: replace Object.keys typecasting when it can be derived as a type
    // https://github.com/microsoft/TypeScript/pull/12253#issuecomment-263132208
    return Object.keys(flags)
        .filter((flagKey) => flags[flagKey].hasOwnProperty('default'))
        .reduce((acc, curr) => {
        return {
            ...acc,
            [curr]: flags[curr].default,
        };
    }, {});
};
const normalizeSandboxOption = (noSandbox, logger) => {
    let sandboxDisabled = false;
    if (noSandbox) {
        if (os.platform() !== 'linux') {
            logger.warn('Disabling sandbox is only relevant on Linux platforms, request declined!');
        }
        else {
            sandboxDisabled = true;
        }
    }
    return {
        noSandbox: sandboxDisabled,
    };
};
export default {
    normalizeOnlyFlagPairs,
    normalizeOutput,
    getDefaultOptions,
    normalizeSandboxOption,
};
//# sourceMappingURL=flags.js.map